# -*- coding: utf-8 -*-
import sys

from qtpy.QtWidgets import QApplication

from ..catalog import Catalog
from .ui import UI

__all__ = ['UI', 'run']


def run() -> int:
    app: QApplication = QApplication(sys.argv)
    window: UI = UI(Catalog(*sys.argv[1:]))
    window.show()
    return app.exec()



# -*- coding: utf-8 -*-
from __future__ import annotations

from qtpy.QtGui import QIcon
from qtpy.QtWidgets import QAction, QMenu, QMenuBar, QStyle, QWidget

__all__ = ['MenuBar']


class MenuBar(QMenuBar):
    def __init__(self, parent: QWidget | None = None) -> None:
        super().__init__(parent)
        self.menu_file: QMenu = QMenu(self.tr('&File'), self)
        self.menu_help: QMenu = QMenu(self.tr('&Help'), self)
        self.menu_edit: QMenu = QMenu(self.tr('&Edit'), self)
        self.menu_columns: QMenu = QMenu(self.tr('&Columns'), self)
        self.menu_copy_only: QMenu = QMenu(self.tr('Copy &Only'), self.menu_edit)
        self.action_load: QAction = QAction(
            QIcon.fromTheme('document-open', self.style().standardIcon(QStyle.StandardPixmap.SP_DialogOpenButton)),
            self.tr('&Load Catalog...'),
            self.menu_file)
        self.action_reload: QAction = QAction(
            QIcon.fromTheme('document-revert', self.style().standardIcon(QStyle.StandardPixmap.SP_BrowserReload)),
            self.tr('&Reload Catalogs'),
            self.menu_file)
        self.action_download_catalog: QAction = QAction(self.tr('&Download Catalog...'), self.menu_file)
        self.action_preferences: QAction = QAction(self.tr('&Preferences...'), self.menu_file)
        self.action_quit: QAction = QAction(QIcon.fromTheme('application-exit'), self.tr('&Quit'),
                                            self.menu_file)
        self.action_check_updates: QAction = QAction(self.tr('Check for Updates...'), self.menu_help)
        self.action_about_catalogs: QAction = QAction(
            QIcon.fromTheme('help-about', self.style().standardIcon(QStyle.StandardPixmap.SP_FileDialogInfoView)),
            self.tr('About Catalogs...'),
            self.menu_help)
        self.action_about: QAction = QAction(
            QIcon.fromTheme('help-about', self.style().standardIcon(QStyle.StandardPixmap.SP_FileDialogInfoView)),
            self.tr('&About...'),
            self.menu_help)
        self.action_about_qt: QAction = QAction(self.style().standardIcon(QStyle.StandardPixmap.SP_TitleBarMenuButton),
                                                self.tr('About &Qt...'), self.menu_help)
        self.action_copy: QAction = QAction(QIcon.fromTheme('edit-copy'), self.tr('Co&py Selection'),
                                            self.menu_edit)
        self.action_clear: QAction = QAction(
            QIcon.fromTheme('edit-clear', self.style().standardIcon(QStyle.StandardPixmap.SP_DialogResetButton)),
            self.tr('&Clear Results'),
            self.menu_edit)
        self.action_select_all: QAction = QAction(QIcon.fromTheme('edit-select-all'), self.tr('&Select All'),
                                                  self.menu_edit)
        self.action_copy_name: QAction = QAction(self.tr('&Substance Name'), self.menu_copy_only)
        self.action_copy_frequency: QAction = QAction(self.tr('&Frequency'), self.menu_copy_only)
        self.action_copy_intensity: QAction = QAction(self.tr('&Intensity'), self.menu_copy_only)
        self.action_copy_lower_state_energy: QAction = QAction(self.menu_copy_only.tr('&Lower State Energy'),
                                                               self.menu_copy_only)
        self.action_show_frequency: QAction = QAction(self.tr('&Frequency'), self.menu_columns)
        self.action_show_intensity: QAction = QAction(self.tr('&Intensity'), self.menu_columns)
        self.action_show_lower_state_energy: QAction = QAction(self.tr('&Lower State Energy'),
                                                               self.menu_columns)
        self.action_substance_info: QAction = QAction(self.tr('Substance &Info'), self.menu_edit)

        self.action_preferences.setMenuRole(QAction.MenuRole.PreferencesRole)
        self.action_quit.setMenuRole(QAction.MenuRole.QuitRole)
        self.action_about.setMenuRole(QAction.MenuRole.AboutRole)
        self.action_about_qt.setMenuRole(QAction.MenuRole.AboutQtRole)
        self.menu_file.addAction(self.action_load)
        self.menu_file.addAction(self.action_reload)
        self.menu_file.addSeparator()
        self.menu_file.addAction(self.action_download_catalog)
        self.menu_file.addSeparator()
        self.menu_file.addAction(self.action_preferences)
        self.menu_file.addSeparator()
        self.menu_file.addAction(self.action_quit)
        self.menu_help.addAction(self.action_check_updates)
        self.menu_help.addSeparator()
        self.menu_help.addAction(self.action_about_catalogs)
        self.menu_help.addAction(self.action_about)
        self.menu_help.addAction(self.action_about_qt)
        self.menu_copy_only.addAction(self.action_copy_name)
        self.menu_copy_only.addAction(self.action_copy_frequency)
        self.menu_copy_only.addAction(self.action_copy_intensity)
        self.menu_copy_only.addAction(self.action_copy_lower_state_energy)
        self.menu_edit.addAction(self.action_clear)
        self.menu_edit.addSeparator()
        self.menu_edit.addAction(self.menu_copy_only.menuAction())
        self.menu_edit.addAction(self.action_copy)
        self.menu_edit.addSeparator()
        self.menu_edit.addAction(self.action_select_all)
        self.menu_edit.addSeparator()
        self.menu_edit.addAction(self.action_substance_info)
        self.menu_columns.addAction(self.action_show_frequency)
        self.menu_columns.addAction(self.action_show_intensity)
        self.menu_columns.addAction(self.action_show_lower_state_energy)
        self.addAction(self.menu_file.menuAction())
        self.addAction(self.menu_edit.menuAction())
        self.addAction(self.menu_columns.menuAction())
        self.addAction(self.menu_help.menuAction())

        self.action_load.setShortcut('Ctrl+L')
        self.action_quit.setShortcut('Ctrl+Q')
        self.action_about.setShortcut('F1')
        self.action_preferences.setShortcut('Ctrl+,')
        self.action_copy.setShortcut('Ctrl+C')
        self.action_select_all.setShortcut('Ctrl+A')
        self.action_reload.setShortcut('Ctrl+R')
        self.action_copy_name.setShortcut('Ctrl+Shift+C, N')
        self.action_copy_frequency.setShortcut('Ctrl+Shift+C, F')
        self.action_copy_intensity.setShortcut('Ctrl+Shift+C, I')
        self.action_copy_lower_state_energy.setShortcut('Ctrl+Shift+C, E')
        self.action_substance_info.setShortcut('Ctrl+I')
        self.action_show_frequency.setCheckable(True)
        self.action_show_intensity.setCheckable(True)
        self.action_show_lower_state_energy.setCheckable(True)



# -*- coding: utf-8 -*-
from __future__ import annotations

from functools import partial

from qtpy.QtWidgets import (QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDoubleSpinBox, QFormLayout, QGroupBox,
                            QSpinBox, QVBoxLayout, QWidget)

from .settings import Settings

__all__ = ['Preferences']


class Preferences(QDialog):
    """ GUI preferences dialog """

    def __init__(self, settings: Settings, parent: QWidget | None = None) -> None:
        super().__init__(parent)

        self.settings: Settings = settings
        self.setModal(True)
        self.setWindowTitle(self.tr('Preferences'))
        if parent is not None:
            self.setWindowIcon(parent.windowIcon())

        layout: QVBoxLayout = QVBoxLayout(self)
        check_box: QCheckBox
        combo_box: QComboBox
        spin_box: QSpinBox | QDoubleSpinBox
        for key, value in self.settings.DIALOG.items():
            if not (isinstance(value, dict) and value):
                continue
            box: QGroupBox = QGroupBox(key, self)
            box_layout: QFormLayout = QFormLayout(box)
            for key2, value2 in value.items():
                if isinstance(value2, tuple) and isinstance(value2[-1], str) and value2[-1]:
                    if len(value2) == 1:
                        check_box = QCheckBox(self.tr(key2), box)
                        setattr(check_box, 'callback', value2[-1])
                        check_box.setChecked(getattr(self.settings, value2[-1]))
                        check_box.toggled.connect(partial(self._on_event, sender=check_box))
                        box_layout.addWidget(check_box)
                    elif len(value2) == 2:
                        value3 = value2[0]
                        if isinstance(value3, (list, tuple)):
                            combo_box = QComboBox(box)
                            setattr(combo_box, 'callback', value2[-1])
                            for item in value3:
                                combo_box.addItem(self.tr(item))
                            combo_box.setCurrentIndex(getattr(self.settings, value2[-1]))
                            combo_box.currentIndexChanged.connect(
                                partial(self._on_combo_box_current_index_changed, sender=combo_box))
                            box_layout.addRow(self.tr(key2), combo_box)
                        # no else
                    elif len(value2) == 3:
                        value3a = value2[0]
                        value3b = value2[1]
                        if isinstance(value3a, (list, tuple)) and isinstance(value3b, (list, tuple)):
                            combo_box = QComboBox(box)
                            setattr(combo_box, 'callback', value2[-1])
                            for index, item in enumerate(value3a):
                                combo_box.addItem(self.tr(item), value3b[index])
                            combo_box.setCurrentIndex(value3b.index(getattr(self.settings, value2[-1])))
                            combo_box.currentIndexChanged.connect(
                                partial(self._on_combo_box_current_index_changed, sender=combo_box))
                            box_layout.addRow(self.tr(key2), combo_box)
                        elif (isinstance(value3a, slice)
                              and isinstance(getattr(self.settings, value2[-1]), (int, float))
                              and isinstance(value3b, tuple)):
                            if ((value3a.start is None or isinstance(value3a.start, int))
                                    and (value3a.stop is None or isinstance(value3a.stop, int))
                                    and (value3a.step is None or isinstance(value3a.step, int))
                                    and isinstance(getattr(self.settings, value2[-1]), int)):
                                spin_box = QSpinBox(box)
                            else:
                                spin_box = QDoubleSpinBox(box)
                            setattr(spin_box, 'callback', value2[-1])
                            if value3a.start is not None:
                                spin_box.setMinimum(value3a.start)
                            if value3a.stop is not None:
                                spin_box.setMaximum(value3a.stop)
                            if value3a.step is not None:
                                spin_box.setSingleStep(value3a.step)
                            spin_box.setValue(getattr(self.settings, value2[-1]))
                            if len(value3b) == 2:
                                spin_box.setPrefix(str(value3b[0]))
                                spin_box.setSuffix(str(value3b[1]))
                            elif len(value3b) == 1:
                                spin_box.setSuffix(str(value3b[0]))
                            # no else
                            spin_box.valueChanged.connect(partial(self._on_event, sender=spin_box))
                            box_layout.addRow(self.tr(key2), spin_box)
                        # no else
                    # no else
                # no else
                layout.addWidget(box)
        buttons: QDialogButtonBox = QDialogButtonBox(QDialogButtonBox.StandardButton.Close, self)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

    # https://forum.qt.io/post/671245
    def _on_event(self, x: bool | int | float, sender: QWidget) -> None:
        setattr(self.settings, getattr(sender, 'callback'), x)

    def _on_combo_box_current_index_changed(self, _: int, sender: QComboBox) -> None:
        setattr(self.settings, getattr(sender, 'callback'), sender.currentData())



# -*- coding: utf-8 -*-
from __future__ import annotations

import os
from typing import Any, Callable, Final

from qtpy.QtCore import QSettings

from ..utils import *

__all__ = ['Settings']


class Settings(QSettings):
    """ convenient internal representation of the application settings """
    FREQUENCY_UNITS: Final[list[str]] = ['MHz', 'GHz', 'cm⁻¹', 'nm']
    INTENSITY_UNITS: Final[list[str]] = ['lg(nm² × MHz)', 'nm² × MHz', 'lg(cm / molecule)', 'cm / molecule']
    ENERGY_UNITS: Final[list[str]] = ['cm⁻¹', 'meV', 'J']
    TEMPERATURE_UNITS: Final[list[str]] = ['K', '°C']
    LINE_ENDS: Final[list[str]] = [r'Line Feed (\n)', r'Carriage Return (\r)', r'CR+LF (\r\n)', r'LF+CR (\n\r)']
    _LINE_ENDS: Final[list[str]] = ['\n', '\r', '\r\n', '\n\r']
    CSV_SEPARATORS: Final[list[str]] = [r'comma (,)', r'tab (\t)', r'semicolon (;)', r'space ( )']
    _CSV_SEPARATORS: Final[list[str]] = [',', '\t', ';', ' ']

    DIALOG: dict[str, dict[str, tuple[Any, ...]]] = {
        'When the program starts': {
            'Load catalogs': ('load_last_catalogs',),
            'Check for update': ('check_updates',),
        },
        'Display': {
            'Allow rich text in formulas': ('rich_text_in_formulas',),
        },
        'Search': {
            'Timeout:': (slice(1, 99), (' sec',), 'timeout',),
        },
        'Units': {
            'Frequency:': (FREQUENCY_UNITS, 'frequency_unit'),
            'Intensity:': (INTENSITY_UNITS, 'intensity_unit'),
            'Energy:': (ENERGY_UNITS, 'energy_unit'),
            'Temperature:': (TEMPERATURE_UNITS, 'temperature_unit'),
        },
        'Export': {
            'With units': ('with_units',),
            'Line ending:': (LINE_ENDS, _LINE_ENDS, 'line_end'),
            'CSV separator:': (CSV_SEPARATORS, _CSV_SEPARATORS, 'csv_separator'),
        }
    }

    TO_MHZ: Final[list[Callable[[float], float]]] = [lambda x: x, ghz_to_mhz, rec_cm_to_mhz, nm_to_mhz]
    FROM_MHZ: Final[list[Callable[[float], float]]] = [lambda x: x, mhz_to_ghz, mhz_to_rec_cm, mhz_to_nm]

    TO_LOG10_SQ_NM_MHZ: Final[list[Callable[[float], float]]] = [
        lambda x: x,
        sq_nm_mhz_to_log10_sq_nm_mhz,
        log10_cm_per_molecule_to_log10_sq_nm_mhz,
        cm_per_molecule_to_log10_sq_nm_mhz
    ]
    FROM_LOG10_SQ_NM_MHZ: Final[list[Callable[[float], float]]] = [
        lambda x: x,
        log10_sq_nm_mhz_to_sq_nm_mhz,
        log10_sq_nm_mhz_to_log10_cm_per_molecule,
        log10_sq_nm_mhz_to_cm_per_molecule
    ]

    TO_REC_CM: Final[list[Callable[[float], float]]] = [
        lambda x: x,
        meV_to_rec_cm,
        j_to_rec_cm
    ]
    FROM_REC_CM: Final[list[Callable[[float], float]]] = [
        lambda x: x,
        rec_cm_to_meV,
        rec_cm_to_j
    ]

    TO_K: Final[list[Callable[[float], float]]] = [lambda x: x, lambda x: x + 273.15]
    FROM_K: Final[list[Callable[[float], float]]] = [lambda x: x, lambda x: x - 273.15]

    def __init__(self, *args: Any) -> None:
        super().__init__(*args)

    @property
    def frequency_unit(self) -> int:
        self.beginGroup('frequency')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return v

    @frequency_unit.setter
    def frequency_unit(self, new_value: int | str) -> None:
        self.beginGroup('frequency')
        if isinstance(new_value, str):
            new_value = self.FREQUENCY_UNITS.index(new_value)
        self.setValue('unit', new_value)
        self.endGroup()

    @property
    def frequency_unit_str(self) -> str:
        self.beginGroup('frequency')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.FREQUENCY_UNITS[v]

    @property
    def to_mhz(self) -> Callable[[float], float]:
        self.beginGroup('frequency')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.TO_MHZ[v]

    @property
    def from_mhz(self) -> Callable[[float], float]:
        self.beginGroup('frequency')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.FROM_MHZ[v]

    @property
    def intensity_unit(self) -> int:
        self.beginGroup('intensity')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return v

    @intensity_unit.setter
    def intensity_unit(self, new_value: int | str) -> None:
        self.beginGroup('intensity')
        if isinstance(new_value, str):
            new_value = self.INTENSITY_UNITS.index(new_value)
        self.setValue('unit', new_value)
        self.endGroup()

    @property
    def intensity_unit_str(self) -> str:
        self.beginGroup('intensity')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.INTENSITY_UNITS[v]

    @property
    def to_log10_sq_nm_mhz(self) -> Callable[[float], float]:
        self.beginGroup('intensity')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.TO_LOG10_SQ_NM_MHZ[v]

    @property
    def from_log10_sq_nm_mhz(self) -> Callable[[float], float]:
        self.beginGroup('intensity')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.FROM_LOG10_SQ_NM_MHZ[v]

    @property
    def energy_unit(self) -> int:
        self.beginGroup('energy')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return v

    @energy_unit.setter
    def energy_unit(self, new_value: int | str) -> None:
        self.beginGroup('energy')
        if isinstance(new_value, str):
            new_value = self.ENERGY_UNITS.index(new_value)
        self.setValue('unit', new_value)
        self.endGroup()

    @property
    def energy_unit_str(self) -> str:
        self.beginGroup('energy')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.ENERGY_UNITS[v]

    @property
    def to_rec_cm(self) -> Callable[[float], float]:
        self.beginGroup('energy')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.TO_REC_CM[v]

    @property
    def from_rec_cm(self) -> Callable[[float], float]:
        self.beginGroup('energy')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.FROM_REC_CM[v]

    @property
    def temperature_unit(self) -> int:
        self.beginGroup('temperature')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return v

    @temperature_unit.setter
    def temperature_unit(self, new_value: int | str) -> None:
        self.beginGroup('temperature')
        if isinstance(new_value, str):
            new_value = self.TEMPERATURE_UNITS.index(new_value)
        self.setValue('unit', new_value)
        self.endGroup()

    @property
    def temperature_unit_str(self) -> str:
        self.beginGroup('temperature')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.TEMPERATURE_UNITS[v]

    @property
    def to_k(self) -> Callable[[float], float]:
        self.beginGroup('temperature')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.TO_K[v]

    @property
    def from_k(self) -> Callable[[float], float]:
        self.beginGroup('temperature')
        v: int = self.value('unit', 0, int)
        self.endGroup()
        return self.FROM_K[v]

    @property
    def load_last_catalogs(self) -> bool:
        self.beginGroup('start')
        v: bool = self.value('loadLastCatalogs', True, bool)
        self.endGroup()
        return v

    @load_last_catalogs.setter
    def load_last_catalogs(self, new_value: bool) -> None:
        self.beginGroup('start')
        self.setValue('loadLastCatalogs', new_value)
        self.endGroup()

    @property
    def check_updates(self) -> bool:
        self.beginGroup('start')
        v: bool = self.value('checkUpdates', True, bool)
        self.endGroup()
        return v

    @check_updates.setter
    def check_updates(self, new_value: bool) -> None:
        self.beginGroup('start')
        self.setValue('checkUpdates', new_value)
        self.endGroup()

    @property
    def rich_text_in_formulas(self) -> bool:
        self.beginGroup('display')
        v: bool = self.value('richTextInFormulas', True, bool)
        self.endGroup()
        return v

    @rich_text_in_formulas.setter
    def rich_text_in_formulas(self, new_value: bool) -> None:
        self.beginGroup('display')
        self.setValue('richTextInFormulas', new_value)
        self.endGroup()

    @property
    def line_end(self) -> str:
        self.beginGroup('export')
        v: int = self.value('lineEnd', self._LINE_ENDS.index(os.linesep), int)
        self.endGroup()
        return self._LINE_ENDS[v]

    @line_end.setter
    def line_end(self, new_value: str) -> None:
        self.beginGroup('export')
        self.setValue('lineEnd', self._LINE_ENDS.index(new_value))
        self.endGroup()

    @property
    def csv_separator(self) -> str:
        self.beginGroup('export')
        v: int = self.value('csvSeparator', self._CSV_SEPARATORS.index('\t'), int)
        self.endGroup()
        return self._CSV_SEPARATORS[v]

    @csv_separator.setter
    def csv_separator(self, new_value: str) -> None:
        self.beginGroup('export')
        self.setValue('csvSeparator', self._CSV_SEPARATORS.index(new_value))
        self.endGroup()

    @property
    def with_units(self) -> bool:
        self.beginGroup('export')
        v: bool = self.value('withUnits', True, bool)
        self.endGroup()
        return v

    @with_units.setter
    def with_units(self, new_value: bool) -> None:
        self.beginGroup('export')
        self.setValue('withUnits', new_value)
        self.endGroup()

    @property
    def timeout(self) -> float:
        self.beginGroup('search')
        v: float = self.value('timeout', 99.0, float)
        self.endGroup()
        return v

    @timeout.setter
    def timeout(self, new_value: float) -> None:
        self.beginGroup('search')
        self.setValue('timeout', new_value)
        self.endGroup()
